"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WinixPurifierPlatform = void 0;
const settings_1 = require("./settings");
const winix_api_1 = require("winix-api");
const accessory_1 = require("./accessory");
const errors_1 = require("./errors");
const DEFAULT_DEVICE_REFRESH_INTERVAL_MINUTES = 60;
class WinixPurifierPlatform {
    constructor(log, platformConfig, api) {
        this.log = log;
        this.platformConfig = platformConfig;
        this.api = api;
        this.Service = this.api.hap.Service;
        this.Characteristic = this.api.hap.Characteristic;
        this.HapStatusError = this.api.hap.HapStatusError;
        this.config = platformConfig;
        this.accessories = new Map();
        this.handlers = new Map();
        this.api.on("didFinishLaunching" /* APIEvent.DID_FINISH_LAUNCHING */, this.onFinishLaunching.bind(this));
    }
    async onFinishLaunching() {
        if (!this.config.auth?.refreshToken) {
            this.log.warn('Winix Purifiers is NOT set up. ' +
                'Please link your Winix account in the Homebridge UI.');
            return;
        }
        try {
            this.winix = await winix_api_1.WinixAccount.fromExistingAuth(this.config.auth);
        }
        catch (e) {
            this.log.error('error generating winix account from existing auth:', e);
            return;
        }
        await this.discoverDevices();
        // refresh devices on the configured interval
        const refreshIntervalMs = (this.config.deviceRefreshIntervalMinutes ?? DEFAULT_DEVICE_REFRESH_INTERVAL_MINUTES) * 60 * 1000;
        setInterval(async () => await this.discoverDevices(), refreshIntervalMs);
    }
    async discoverDevices() {
        this.log.debug('Starting device discovery...');
        let devices;
        try {
            devices = await this.winix?.getDevices();
        }
        catch (e) {
            (0, errors_1.assertError)(e);
            this.log.error('error getting devices:', e.message);
            return;
        }
        // if devices is explicitly typeof undefined, then the user has not logged in yet
        if (typeof devices === 'undefined') {
            this.log.warn('Winix Purifiers is NOT set up. ' +
                'Please log in with your Winix account credentials in the Homebridge UI.');
            return;
        }
        if (devices.length === 0) {
            this.log.warn('No Winix devices found. Please add devices to your Winix account.');
        }
        const accessoriesToAdd = [];
        const discoveredUUIDs = new Set();
        for (const device of devices) {
            const uuid = this.api.hap.uuid.generate(device.deviceId);
            let accessory = this.accessories.get(uuid);
            if (accessory) {
                this.log.debug('Found existing accessory:', logName(device));
                accessory.context.device = device;
                this.api.updatePlatformAccessories([accessory]);
            }
            else {
                this.log.debug('Found new accessory:', logName(device));
                accessory = new this.api.platformAccessory(device.deviceAlias, uuid, 19 /* Categories.AIR_PURIFIER */);
                accessory.context.device = device;
                this.accessories.set(uuid, accessory);
                this.handlers.set(uuid, new accessory_1.WinixPurifierAccessory(this.log, this, this.config, accessory));
                accessoriesToAdd.push(accessory);
            }
            discoveredUUIDs.add(accessory.UUID);
        }
        if (accessoriesToAdd.length > 0) {
            this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, accessoriesToAdd);
        }
        this.removeOldDevices(discoveredUUIDs);
    }
    removeOldDevices(discoveredUUIDs) {
        this.accessories.forEach((accessory) => {
            // if we've seen this accessory in discovered devices, don't remove it
            if (discoveredUUIDs.has(accessory.UUID)) {
                return;
            }
            this.log.debug('Removing old accessory:', logName(accessory.context.device));
            this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
            this.accessories.delete(accessory.UUID);
            this.handlers.delete(accessory.UUID);
        });
    }
    configureAccessory(accessory) {
        this.log.debug('Loading cached accessory:', logName(accessory.context.device));
        this.accessories.set(accessory.UUID, accessory);
        this.handlers.set(accessory.UUID, new accessory_1.WinixPurifierAccessory(this.log, this, this.config, accessory));
    }
}
exports.WinixPurifierPlatform = WinixPurifierPlatform;
function logName(device) {
    return `${device.modelName}-${device.deviceAlias}`;
}
